X-Git-Url: https://git.rapsys.eu/packbundle/blobdiff_plain/ff3c3d6005d4b6263ab27b6541390a9d95d7d822..67d50a43b1bc369544315f6e407db813ad2edf5c:/Util/SluggerUtil.php

diff --git a/Util/SluggerUtil.php b/Util/SluggerUtil.php
index 5b05958..0ff32c5 100644
--- a/Util/SluggerUtil.php
+++ b/Util/SluggerUtil.php
@@ -12,67 +12,51 @@
 namespace Rapsys\PackBundle\Util;
 
 /**
- * Helps manage string conversions
+ * Manages string conversions
  */
 class SluggerUtil {
-	//The secret parameter
-	private $secret;
-
-	//The alpha array
-	private $alpha;
+	/**
+	 * The alpha array
+	 */
+	protected array $alpha;
 
-	//The rev array
-	private $rev;
+	/**
+	 * The rev array
+	 */
+	protected array $rev;
 
-	//The alpha array key number
-	private $count;
+	/**
+	 * The alpha array key number
+	 */
+	protected int $count;
 
-	//The offset reduced from secret
-	private $offset;
+	/**
+	 * The offset reduced from secret
+	 */
+	protected int $offset;
 
 	/**
-	 * Creates a new slugger util
+	 * Construct slugger util
+	 *
+	 * Run "bin/console rapsyspack:range" to generate RAPSYSPACK_RANGE="ayl[...]z9w" range in .env.local
 	 *
-	 * @todo Add a command to generate alpha array or generate it on first run with cache storage ?
-	 * @todo Use Cache like in calendar controller through FilesystemAdapter
+	 * @todo Use Cache like in calendar controller through FilesystemAdapter ?
 	 *
 	 * @param string $secret The secret string
 	 */
-	public function __construct(string $secret) {
-		//Set secret
-		$this->secret = $secret;
+	public function __construct(protected string $secret) {
+		//Without range
+		if (!isset($_ENV['RAPSYSPACK_RANGE']) || empty($range = $_ENV['RAPSYSPACK_RANGE']) || $range === 'Ch4ng3m3!') {
+			//Protect member variable setup
+			return;
+		}
 
 		/**
-		 * Pseudo-random alphabet
-		 * @xxx use array flip and keys to workaround php "smart" that cast range('0', '9') as int instead of string
-		 * @xxx The key count mismatch, count(alpha)>count(rev), resulted in a data corruption due to duplicate numeric values
-		 * @todosee required range by json_encode result and short input (0->255 ???)
+		 * Get pseuto-random alphabet by splitting range string
+		 * TODO: see required range by json_encode result and short input (0->255 ???)
+		 * XXX: The key count mismatch, count(alpha)>count(rev), resulted in a data corruption due to duplicate numeric values
 		 */
-		$this->alpha = array_keys(array_flip(array_merge(
-			range('^', '[', -1),
-			range('V', 'Z'),
-			range('9', '7', -1),
-			range('L', 'O'),
-			range('f', 'a', -1),
-			range('_', '`'),
-			range('3', '0', -1),
-			range('E', 'H'),
-			range('v', 'r', -1),
-			range('+', '/'),
-			range('K', 'I', -1),
-			range('g', 'j'),
-			range('=', ':', -1),
-			range('>', '@'),
-			range('m', 'k', -1),
-			range('4', '6'),
-			range('*', '%', -1),
-			range('n', 'q'),
-			range('U', 'P', -1),
-			range(' ', '$'),
-			range('D', 'A', -1),
-			range('w', 'z'),
-			range('~', '!', -1)
-		)));
+		$this->alpha = str_split($range);
 
 		//Init rev array
 		$this->count = count($rev = $this->rev = array_flip($this->alpha));
@@ -81,6 +65,7 @@ class SluggerUtil {
 		$split = str_split($this->secret);
 
 		//Set offset
+		//TODO: protect undefined index ?
 		$this->offset = array_reduce($split, function ($res, $a) use ($rev) { return $res += $rev[$a]; }, count($split)) % $this->count;
 	}
 
@@ -118,14 +103,20 @@ class SluggerUtil {
 	/**
 	 * Crypt and base64uri encode string
 	 *
-	 * @param string $data The data string
+	 * @param array|string $data The data string
 	 * @return string The hashed data
 	 */
-	public function hash(string $data): string {
+	public function hash(array|string $data): string {
+		//With array
+		if (is_array($data)) {
+			//Json encode array
+			$data = json_encode($data);
+		}
+
 		//Return hashed data
 		//XXX: we use hash_hmac with md5 hash
 		//XXX: crypt was dropped because it provided identical signature for string starting with same pattern
-		return str_replace(['+','/'], ['-','_'], base64_encode(hash_hmac('md5', $data, $this->secret, true)));
+		return str_replace(['+','/','='], ['-','_',''], base64_encode(hash_hmac('md5', $data, $this->secret, true)));
 	}
 
 	/**
@@ -164,25 +155,58 @@ class SluggerUtil {
 		}
 
 		//Send result
-		return str_replace(['+','/'], ['-','_'], base64_encode($ret));
+		return str_replace(['+','/','='], ['-','_',''], base64_encode($ret));
 	}
 
 	/**
 	 * Convert string to safe slug
 	 *
 	 * @param string $data The data string
 	 * @return string The slugged data
 	 */
-	function slug(string $data): string {
+	function slug(?string $data, string $separator = '-'): ?string {
+		//With null
+		if ($data === null) {
+			//Return null
+			return $data;
+		}
+
 		//Use Transliterator if available
 		if (class_exists('Transliterator')) {
 			//Convert from any to latin, then to ascii and lowercase
 			$trans = \Transliterator::create('Any-Latin; Latin-ASCII; Lower()');
 			//Replace every non alphanumeric character by dash then trim dash
-			return trim(preg_replace('/[^a-zA-Z0-9]+/', '-', $trans->transliterate($data)), '-');
+			return trim(preg_replace('/[^a-zA-Z0-9]+/', $separator, $trans->transliterate($data)), $separator);
 		}
+
 		//Convert from utf-8 to ascii, replace quotes with space, remove non alphanumericseparator, replace separator with dash and trim dash
-		return trim(preg_replace('/[\/_|+ -]+/', '-', strtolower(preg_replace('/[^a-zA-Z0-9\/_|+ -]/', '', str_replace(['\'', '"'], ' ', iconv('UTF-8', 'ASCII//TRANSLIT', $data))))), '-');
+		return trim(preg_replace('/[\/_|+ -]+/', $separator, strtolower(preg_replace('/[^a-zA-Z0-9\/_|+ -]/', '', str_replace(['\'', '"'], ' ', iconv('UTF-8', 'ASCII//TRANSLIT', $data))))), $separator);
+	}
+
+	/**
+	 * Convert string to latin
+	 *
+	 * @param string $data The data string
+	 * @return ?string The slugged data
+	 */
+	function latin(?string $data): ?string {
+		//With null
+		if ($data === null) {
+			//Return null
+			return $data;
+		}
+
+		//Use Transliterator if available
+		if (class_exists('Transliterator')) {
+			//Convert from any to latin, then to ascii and lowercase
+			$trans = \Transliterator::create('Any-Latin; Latin-ASCII');
+			//Replace every non alphanumeric character by dash then trim dash
+			return trim($trans->transliterate($data));
+		}
+
+		//Convert from utf-8 to ascii
+		return trim(iconv('UTF-8', 'ASCII//TRANSLIT', $data));
 	}
 
 	/**
@@ -164,25 +155,58 @@ class SluggerUtil {
 	 *
 	 * @param string $data The data string
-	 * @return string The slugged data
+	 * @param string $separator The separator string
+	 * @return ?string The slugged data
 	 */